//Mugurel Ionut Andreica
//Complexitate: O (N^2 * 2^N + N * K^2 * log K)
#include <stdio.h>
#include <vector>
#include <set>
#include <algorithm>

using namespace std;

#define NMAX 18
#define KMAX 1001
#define INF 1000000000000000LL

#define filein "campanie.in"
#define fileout "campanie.out"

//#define PRINT_DEBUG_INFO

#define min(a, b) ((a) <= (b) ? (a) : (b))

set<long long> ycoords;
set<long long>::iterator it;
long long xas[NMAX], yas[NMAX], xap[NMAX], yap[NMAX], cost[NMAX];
long long di[KMAX], y[2][KMAX], cmin[2];
long long hmin;
int ny[2];
int N;

void process_city(int city) {
	int i, j, k, c, ac;
	
	// Citeste numarul de strazi din orasul city.
	scanf("%d", &k);

	#ifdef PRINT_DEBUG_INFO
	fprintf(stderr, "city=%d, nr strazi=%d\n", city, k);
	#endif

	// Citeste distantele pe orizontala dintre strazi.
	for (i = 1; i < k; i++) {
		scanf("%lld", &di[i]);
		#ifdef PRINT_DEBUG_INFO
		fprintf(stderr, "city=%d, di[%d]=%lld\n", city, i, di[i]);
		#endif
	}

	for (i = 0; i < k; i++) {
		// Citeste cate puncte sunt pe strada i.
		scanf("%d", &ny[c = (i % 2)]);
		#ifdef PRINT_DEBUG_INFO
		fprintf(stderr, "city=%d, strada=%d, nr puncte=%d\n", city, i, ny[c]);
		#endif

		// Citeste punctele de pe strada i.
		for (j = 0; j < ny[c]; j++) {
			scanf("%lld", &y[c][j]);
			#ifdef PRINT_DEBUG_INFO
			fprintf(stderr, "city=%d, strada=%d, ypunct[%d]=%lld\n", city, i, j, y[c][j]);
			#endif
		}

		// Calculeaza costul minim de a ajunge pe strada i.
		if (i == 0) {
			cmin[c] = 0;
		} else {
			ycoords.clear();
			ac = (1 + c) % 2;
			for (j = 0; j < ny[ac]; j++)
				ycoords.insert(y[ac][j]);

			long long d, dmin = INF;
			for (j = 0; j < ny[c]; j++) {
				it = ycoords.lower_bound(y[c][j]);

				if (it != ycoords.end()) {
					d = (y[c][j] - *it) * (y[c][j] - *it) + di[i] * di[i];
					dmin = min(d, dmin);
					#ifdef PRINT_DEBUG_INFO
					//if (city == 0 && i == 3)
					fprintf(stderr, "city=%d, strada=%d, ypunct[%d]=%lld, ypunctanterior=%lld, d=%lld\n", city, i, j, y[c][j], *it, d);
					#endif
				}
				
				if (it != ycoords.begin()) {
					it--;
					d = (y[c][j] - *it) * (y[c][j] - *it) + di[i] * di[i];
					dmin = min(d, dmin);
					#ifdef PRINT_DEBUG_INFO
					//if (city == 0 && i == 3)
					fprintf(stderr, "city=%d, strada=%d, ypunct[%d]=%lld, ypunctanterior=%lld, d=%lld\n", city, i, j, y[c][j], *it, d);
					#endif
				}
			}

			cmin[c] = cmin[ac] + dmin;

			#ifdef PRINT_DEBUG_INFO
			fprintf(stderr, "city=%d, strada=%d, dmin=%lld, cmin=%lld\n", city, i, dmin, cmin[c]);
			#endif
		}
	}
	
	cost[city] = cmin[c];
	#ifdef PRINT_DEBUG_INFO
	fprintf(stderr, "city=%d, cost=%lld\n", city, cost[city]);
	#endif
}

void read_input() {
	int i;
	freopen(filein, "r", stdin);
	scanf("%d", &N);
	for (i = 0; i < N; i++) {
		scanf("%lld %lld %lld %lld", &xas[i], &yas[i], &xap[i], &yap[i]);
		process_city(i);
	}
}

int bit[NMAX];
long long dcity[NMAX][NMAX];
long long h[1 << NMAX][NMAX];

void compute_ham_cycle() {
	int i, j, k, conf, lim = (1 << N) - 1;

	bit[0] = 1;
	for (i = 1; i < N; i++)
		bit[i] = bit[i - 1] << 1;

	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			dcity[i][j] = (xap[i] - xas[j]) * (xap[i] - xas[j]) + (yap[i] - yas[j]) * (yap[i] - yas[j]);
			#ifdef PRINT_DEBUG_INFO
			fprintf(stderr, "dcity(%d,%d)=%lld\n", i, j, dcity[i][j]);
			#endif
		}
	}

	hmin = INF;
	for (i = 0; i < 1; i++) {
		for (j = 0; j < N; j++)
			h[bit[i]][j] = INF;
		h[bit[i]][i] = 0;
		
		for (conf = bit[i] + 1; conf <= lim; conf++) {
			if ((conf & bit[i]) == 0)
				continue;
			for (j = 0; j < N; j++) {
				h[conf][j] = INF;
				if (((conf & bit[j]) > 0) && (i != j)) {
					for (k = 0; k < N; k++)
						if (((conf & bit[k]) > 0) && (k != j))
							h[conf][j] = min(h[conf][j], h[conf - bit[j]][k] + dcity[k][j]);
				}
			}
		}
		
		for (j = 0; j < N; j++) {
			hmin = min(hmin, h[lim][j] + dcity[j][i]);
			#ifdef PRINT_DEBUG_INFO
			fprintf(stderr, "ham cycle: i=%d, j=%d, path_cost=%lld, cycle_cost=%lld\n", i, j, h[lim][j], h[lim][j] + dcity[j][i]);
			#endif
		}
	}
	
	fprintf(stderr, "hmin=%lld\n", hmin);
}

void write_output() {
	int i;
	long long sum = hmin;
	for (i = 0; i < N; i++)
		sum += cost[i];
	
	freopen(fileout, "w", stdout);
	printf("%lld\n", sum);
	
	fprintf(stderr, "Raspuns=%lld\n", sum);
}

int main() {
	read_input();
	compute_ham_cycle();
	write_output();
	return 0;
}
