// Mugurel Ionut Andreica
// O(N * (N/3 + N/4 + ... + N/N)) = aprox. O(N^2 * log(N)) - 50p

#include <stdio.h>
#include <time.h>

#define NMAX 1001
#define P 200003

long long comb[NMAX][NMAX], repcomb[NMAX][NMAX], repcomb2[NMAX][NMAX];
long long fact[NMAX], factk[NMAX][NMAX];
long long matcnt[NMAX][NMAX], summatcnt[NMAX][NMAX];
long long N, inv2, inv2k[NMAX];

void read_input() {
	freopen("matcnt.in", "r", stdin);
	scanf("%lld", &N);
}

void compute_comb() {
	int i, j;
	comb[1][0] = comb[1][1] = 1 % P;
	for (i = 2; i <= N; i++) {
		comb[i][0] = 1 % P;
		for (j = 1; j <= i; j++) {
			comb[i][j] = (comb[i - 1][j - 1] + comb[i - 1][j]) % P;
			//fprintf(stderr, "comb[%d][%d]=%d\n", i, j, comb[i][j]);
		}
	}
}

void compute_repcomb() {
	int i, j;
	for (i = 1; i <= N; i++) {
		for (j = 1; j < i; j++)
			if (i % j == 0) {
				repcomb[i][j] = (comb[i][j] * repcomb[i - j][j]) % P;
				//fprintf(stderr, "repcomb[%d][%d]=%d\n", i, j, repcomb[i][j]);
			}
		repcomb[i][i] = 1 % P;

		for (j = 1; j < i; j++)
			repcomb2[i][j] = (comb[i - 1][j - 1] * repcomb2[i - j][j]) % P;
		repcomb2[i][i] = 1 % P;
	}
}

void compute_fact() {
	int i;
	fact[0] = fact[1] = 1 % P;
	for (i = 2; i <= N; i++) {
		fact[i] = (fact[i - 1] * i) % P;
		//fprintf(stderr, "fact[%d]=%d\n", i, fact[i]);
	}
}

void compute_factk() {
	int i, j;
	for (i = 0; i <= N; i++) {
		factk[i][0] = 1 % P;
		factk[i][1] = fact[i];
		for (j = 2; j <= N; j++)
			factk[i][j] = (factk[i][j - 1] * fact[i]) % P;
	}
}

void compute_inv2() {
	int i;
	for (inv2 = 1; inv2 < P; inv2++)
		if ((inv2 * 2) % P == 1)
			break;
	//fprintf(stderr, "inv2=%d\n", inv2);
	inv2k[1] = inv2;
	for (i = 2; i <= N; i++)
		inv2k[i] = (inv2k[i - 1] * inv2) % P;
}

void compute_matcnt() {
	long long i, j, k, x;

	matcnt[0][0] = 1 % P;
	for (j = 0; j <= N; j++)
		summatcnt[0][j] = 1 % P;

	for (i = 3; i <= N; i++) {
		for (j = 3; j <= i; j++) {
			for (k = 1; k * j <= i; k++) {
				x = summatcnt[i - k * j][j - 1];

				x = (x * comb[i][k * j]) % P;
				x = (x * comb[i][k * j]) % P;
				x = (x * repcomb2[k * j][j]) % P;
				x = (x * repcomb[k * j][j]) % P;

				x = (x * factk[j][k]) % P;
				x = (x * factk[j - 1][k]) % P;
				x = (x * inv2k[k]) % P;
				matcnt[i][j] = (matcnt[i][j] + x) % P;
			}

			//fprintf(stderr, "matcnt[%d][%d]=%d\n", i, j, matcnt[i][j]);
			summatcnt[i][j] = (summatcnt[i][j - 1] + matcnt[i][j]) % P;
		}

		for (j = i + 1; j <= N; j++)
			summatcnt[i][j] = summatcnt[i][j - 1];
	}
}

void write_output() {
	freopen("matcnt.out", "w", stdout);
	printf("%lld\n", summatcnt[N][N]);
	//fprintf(stderr, "N=%d, result=%d\n", N, summatcnt[N][N]);
}

int main() {
	//int tstart = clock();

	read_input();
	compute_comb();
	compute_repcomb();
	compute_fact();
	compute_factk();
	compute_inv2();
	compute_matcnt();
	write_output();

	//fprintf(stderr, "Duration=%.3lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);
	return 0;
}
