/**
 * (C) 2012 Bogdan-Cristian Tătăroiu
 */
#include <cstdio>
#include <cassert>
#include <algorithm>
#include <set>

using namespace std;

const int MAXN = 40000;
const int MAXM = 400000;
const int K = 75;

int N, M;
int endPair[MAXN];
int con[MAXN][2], conK[MAXN][2];

inline int walk(int nod, int dir, int count, bool noSkipping = false) {
    int prv;
    if (!noSkipping) {
        prv = conK[nod][1 ^ dir];
        for (; count >= K && nod != -1; count -= K) {
            int nxt = conK[nod][0] ^ conK[nod][1] ^ prv;
            prv = nod;
            nod = nxt;
            dir = conK[nod][0] == prv;
        }
    }

    prv = con[nod][1 ^ dir];
    for (; count > 0 && nod != -1; count -= 1) {
        int nxt = con[nod][0] ^ con[nod][1] ^ prv;
        prv = nod;
        nod = nxt;
        dir = con[nod][0] == prv;
    }
    return nod;
}

inline int getEnd(int nod, int dir) {
    int prv = conK[nod][1 ^ dir];
    for (; ; ) {
        int nxt = conK[nod][0] ^ conK[nod][1] ^ prv;
        if (nxt == -1) {
            break;
        }
        prv = nod;
        nod = nxt;
        dir = conK[nod][0] == prv;
    }
    prv = con[nod][1 ^ dir];
    for (; ; ) {
        int nxt = con[nod][0] ^ con[nod][1] ^ prv;
        if (nxt == -1) {
            break;
        }
        prv = nod;
        nod = nxt;
        dir = con[nod][0] == prv;
    }
    return nod;
}

inline void addEdge(int X, int Y) {
    int XK[K], XKdir[K], YK[K], YKdir[K];
    int nod, dir, prv;
    nod = X; dir = con[nod][0] == -1; prv = con[nod][1 ^ dir];
    for (int i = 0; i < K; i++) {
        XK[i] = nod;
        if (nod != -1) {
            int nxt = con[nod][0] ^ con[nod][1] ^ prv;
            dir = con[nod][1] == nxt;
            XKdir[i] = 1 ^ dir;
            prv = nod;
            nod = nxt;
        }
    }

    nod = Y; dir = con[nod][0] == -1; prv = con[nod][1 ^ dir];
    for (int i = 0; i < K; i++) {
        YK[i] = nod;
        if (nod != -1) {
            int nxt = con[nod][0] ^ con[nod][1] ^ prv;
            dir = con[nod][1] == nxt;
            YKdir[i] = 1 ^ dir;
            prv = nod;
            nod = nxt;
        }
    }

    for (int i = 0; i < K; i++) {
        if (XK[i] != -1) {
            assert(conK[XK[i]][XKdir[i]] == -1);
            conK[XK[i]][XKdir[i]] = YK[K - i - 1];
        }

        if (YK[i] != -1) {
            assert(conK[YK[i]][YKdir[i]] == -1);
            conK[YK[i]][YKdir[i]] = XK[K - i - 1];
        }
    }

    con[X][XKdir[0]] = Y;
    con[Y][YKdir[0]] = X;
}

inline void removeEdge(int X, int Y) {
    int XK[K], XKdir[K], YK[K], YKdir[K];
    int nod, dir, prv;
    nod = X; dir = con[nod][0] == Y; prv = con[nod][1 ^ dir];
    for (int i = 0; i < K; i++) {
        XK[i] = nod;
        if (nod != -1) {
            int nxt = con[nod][0] ^ con[nod][1] ^ prv;
            dir = con[nod][1] == nxt;
            XKdir[i] = 1 ^ dir;
            prv = nod;
            nod = nxt;
        }
    }

    nod = Y; dir = con[nod][0] == X; prv = con[nod][1 ^ dir];
    for (int i = 0; i < K; i++) {
        YK[i] = nod;
        if (nod != -1) {
            int nxt = con[nod][0] ^ con[nod][1] ^ prv;
            dir = con[nod][1] == nxt;
            YKdir[i] = 1 ^ dir;
            prv = nod;
            nod = nxt;
        }
    }

    for (int i = 0; i < K; i++) {
        if (XK[i] != -1) {
            conK[XK[i]][XKdir[i]] = -1;
        }
        if (YK[i] != -1) {
            conK[YK[i]][YKdir[i]] = -1;
        }
    }

    con[X][XKdir[0]] = -1;
    con[Y][YKdir[0]] = -1;
}

int main() {
    assert(freopen("drumuri.in", "rt", stdin));
#ifndef DEBUG
    assert(freopen("drumuri.out", "wt", stdout));
#endif

    assert(scanf("%d %d", &N, &M) == 2);
    assert(1 <= N && N <= MAXN);
    assert(1 <= M && M <= MAXM);
    for (int i = 0; i < N; i++) {
        endPair[i] = i;
        con[i][0] = con[i][1] = -1;
        conK[i][0] = conK[i][1] = -1;
    }

    for (; M; M--) {
        int type;
        assert(scanf("%d", &type) == 1);
        assert(1 <= type && type <= 4);
        if (type == 1) {
            int X, Y;
            assert(scanf("%d %d", &X, &Y) == 2);
            X -= 1; Y -= 1;
            assert(0 <= X && X < N && 0 <= Y && Y < N);

            if (con[X][0] != -1 && con[X][1] != -1) {
                printf("0\n");
                continue;
            }
            if (con[Y][0] != -1 && con[Y][1] != -1) {
                printf("0\n");
                continue;
            }
            if (endPair[X] == Y) {
                assert(endPair[Y] == X);
                printf("0\n");
                continue;
            }

            addEdge(X, Y);

            int endPairX = endPair[X], endPairY = endPair[Y];
            endPair[endPairX] = endPairY;
            endPair[endPairY] = endPairX;
            printf("1\n");
        } else if (type == 2) {
            int X, Y;
            assert(scanf("%d %d", &X, &Y) == 2);
            X -= 1; Y -= 1;
            assert(0 <= X && X < N && 0 <= Y && Y < N);

            if (con[X][0] != Y && con[X][1] != Y) {
                assert(con[Y][0] != X && con[Y][1] != X);
                printf("0\n");
                continue;
            }
            assert(con[Y][0] == X || con[Y][1] == X);

            removeEdge(X, Y);
            endPair[X] = getEnd(X, con[X][0] == -1);
            endPair[Y] = getEnd(Y, con[Y][0] == -1);
            endPair[endPair[X]] = X;
            endPair[endPair[Y]] = Y;
            printf("1\n");
        } else if (type == 3) {
            int nod, dist, n1, n2;
            assert(scanf("%d %d", &nod, &dist) == 2);
            nod -= 1;
            assert(0 <= nod && nod < N);

            n1 = walk(nod, 0, dist);
            n2 = walk(nod, 1, dist);
            set<int> ans;
            if (n1 != -1) {
                ans.insert(n1);
            }
            if (n2 != -1) {
                ans.insert(n2);
            }
            printf("%d", (int)ans.size());
            for (set<int> :: iterator it = ans.begin(); it != ans.end(); it++) {
                printf(" %d", *it + 1);
            }
            printf("\n");
        } else if (type == 4) {
            int nod, n1, n2;
            assert(scanf("%d", &nod) == 1);
            nod -= 1;
            assert(0 <= nod && nod < N);

            n1 = getEnd(nod, 0);
            n2 = getEnd(nod, 1);
            set<int> ans;
            ans.insert(n1);
            ans.insert(n2);
            printf("%d", (int)ans.size());
            for (set<int> :: iterator it = ans.begin(); it != ans.end(); it++) {
                printf(" %d", *it + 1);
            }
            printf("\n");
        }

        //{
        //    for (int i = 0; i < N; i++) {
        //        assert(conK[i][0] == walk(i, 0, K, true));
        //        assert(conK[i][1] == walk(i, 1, K, true));
        //    }
        //    fflush(stdout);
        //}
    }

    return 0;
}
