/* Mugurel Andreica */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <set>
#include <vector>

//#define USE_STL_VECTORS

using namespace std;

#define NMAX 40001

#define myabs(x) ((x) < 0 ? (-(x)) : (x))

set<int> available_positions;

#ifdef USE_STL_VECTORS
vector<int> path[NMAX];
#else
unsigned short int* path[NMAX];
#endif

int path_len[NMAX], path_id[NMAX], path_pos[NMAX];
int N, M, i, j, d, op, x, y, z;

void path_union(int i, int revi, int j, int revj) {
#ifdef USE_STL_VECTORS
	if (path_len[i] < path_len[j]) {
		path_union(j, 1 - revj, i, 1 - revi);
		return;
	}
#endif

	if (revi) {
		for (x = 0, y = path_len[i] - 1; x < y; ++x, --y) {
			z = path[i][x];
			path[i][x] = path[i][y];
			path[i][y] = z;
			path_pos[path[i][x]] = x;
			path_pos[path[i][y]] = y;
		}
	}

#ifdef USE_STL_VECTORS
	if (revj) {
		for (x = path_len[j] - 1; x >= 0; --x) {
			path[i].push_back(path[j][x]);
		}
	} else {
		for (x = 0; x < path_len[j]; ++x) {
			path[i].push_back(path[j][x]);
		}
	}
#else
	path[i] = (unsigned short int*) realloc(path[i], (path_len[i] + path_len[j]) * sizeof(unsigned short int));

	if (revj) {
		for (x = path_len[j] - 1, y = path_len[i]; x >= 0; --x, ++y) {
			path[i][y] = path[j][x];
		}
	} else {
		memcpy(&(path[i][path_len[i]]), path[j], path_len[j] * sizeof(unsigned short int));
	}
#endif
		
	for (x = path_len[i]; x < path_len[i] + path_len[j]; ++x) {
		path_id[path[i][x]] = i;
		path_pos[path[i][x]] = x;
	}
	
	path_len[i] += path_len[j];
	available_positions.insert(j);

#ifdef USE_STL_VECTORS
	path[j].clear();
#else
	delete path[j];
#endif
}

void path_split(int i, int left_len) {
	j = *(available_positions.begin());
	available_positions.erase(j);
	path_len[j] = path_len[i] - left_len;

#ifdef USE_STL_VECTORS
#else
	path[j] = new unsigned short int[path_len[j]];
#endif

	for (x = left_len; x < path_len[i]; ++x) {
#ifdef USE_STL_VECTORS
		path[j].push_back(path[i][x]);
#else
		path[j][x - left_len] = path[i][x];
#endif
		path_id[path[i][x]] = j;
		path_pos[path[i][x]] = x - left_len;
	}

	path_len[i] = left_len;

#ifdef USE_STL_VECTORS
	path[i].resize(left_len);
#else
	path[i] = (unsigned short int*) realloc(path[i], left_len * sizeof(unsigned short int));
#endif
}

void print_nodes(int u, int v) {
	if (u == v)
		u = 0;
	if (u > 0 && v > 0) {
        printf("2 ");
		if (u < v)
			printf("%d %d\n", u, v);
		else
			printf("%d %d\n", v, u);
	} else if (u == 0 && v > 0) {
		printf("1 %d\n", v);
	} else if (u > 0 && v == 0) {
		printf("1 %d\n", u);
	} else {
		printf("0\n");
	}
}

int main() {
	int tstart = clock();
	freopen("drumuri.in", "r", stdin);
	freopen("drumuri.out", "w", stdout);

	scanf("%d %d", &N, &M);
	for (i = 1; i <= N; i++) {
		#ifdef USE_STL_VECTORS
		path[i].push_back(i);
		#else
		path[i] = new unsigned short int[1];
		path[i][0] = i;
		#endif
		path_len[i] = 1;
		path_id[i] = i;
		path_pos[i] = 0;
	}

	while (M--) {
		scanf("%d", &op);
		
		if (op == 1) {
			scanf("%d %d", &i, &j);
			if (path_id[i] != path_id[j] &&
				(path_pos[i] == 0 || path_pos[i] == path_len[path_id[i]] - 1) &&
				(path_pos[j] == 0 || path_pos[j] == path_len[path_id[j]] - 1)) {

				if (path_pos[i] == 0 && path_pos[j] > 0) {
					path_union(path_id[j], 0, path_id[i], 0);
				} else if (path_pos[i] == 0 && path_pos[j] == 0) {
					path_union(path_id[i], 1, path_id[j], 0);
				} else if (path_pos[i] > 0 && path_pos[j] == 0) {
					path_union(path_id[i], 0, path_id[j], 0);
				} else { // path_pos[i] > 0 && path_pos[j] > 0
					path_union(path_id[i], 0, path_id[j], 1);
				}

				printf("1\n");
			} else {
				printf("0\n");
			}
		} else if (op == 2) {
			scanf("%d %d", &i, &j);
			if (path_id[i] == path_id[j] &&
				myabs(path_pos[i] - path_pos[j]) == 1) {
				path_split(path_id[i], min(path_pos[i], path_pos[j]) + 1);
				printf("1\n");
			} else {
				printf("0\n");
			}
		} else if (op == 3) {
			scanf("%d %d", &i, &d);
			if (path_pos[i] >= d)
				x = path[path_id[i]][path_pos[i] - d];
			else
				x = 0;

			if (path_pos[i] + d < path_len[path_id[i]])
				y = path[path_id[i]][path_pos[i] + d];
			else
				y = 0;

			print_nodes(x, y);
		} else {
			scanf("%d", &i);
			print_nodes(path[path_id[i]][0], path[path_id[i]][path_len[path_id[i]] - 1]);
		}
	}
	
	fprintf(stderr, "Duration=%.3lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);	
	return 0;
}
